var Metalsmith  = require('metalsmith');
var collections = require('metalsmith-collections');
var layouts     = require('metalsmith-layouts');
var markdown    = require('metalsmith-markdown');
var permalinks  = require('metalsmith-permalinks');
var ignore      = require('metalsmith-ignore');
var frontmatter = require('metalsmith-matters');
var debug       = require('metalsmith-debug');
var tags        = require('metalsmith-tags');
var gitInfo     = require('./gitInfo');
var dateFilter  = require('nunjucks-date-filter');


var cons     = require('consolidate');
var nunjucks = require('nunjucks');
cons.requires.nunjucks = nunjucks.configure('./src/theme/templates', {watch: false});

var util = require('util')
cons.requires.nunjucks.addFilter('inspect', function (obj) {
  return util.inspect(obj);
});
cons.requires.nunjucks.addFilter('date', dateFilter);
cons.requires.nunjucks.addFilter('date', d => d)
cons.requires.nunjucks.addFilter('paginationList', function(page, count) {
    var curPage = page;
    var pages = [];
    pages.push(curPage);
    for(var i = 0; i < count; i++) {
      curPage = curPage.metadata.prevpage;
      if(curPage === undefined) break;
      pages.unshift(curPage);
    }

    curPage = page;
    for(let i = 0; i < count; i++) {
      curPage = curPage.metadata.nextpage;
      if(curPage === undefined) break;
      pages.push(curPage);
    }
    return pages;
});
cons.requires.nunjucks.addFilter('relURL', function(filename, dir){
  if(filename.substr(0,1) == '/' || filename.match('://')) return filename;
  if(dir.substr(0,1) !== '/') dir = '/' + dir;
  if(filename.match(/\/$/)) return dir+filename;
  return dir+'/'+filename;
})


Metalsmith(__dirname)
  .frontmatter(false)
  .metadata({
    sitename: "OK1KVK.cz",
    siteurl: "https://ok1kvk.cz/",
    description: "Stránky OK1KVK, radioklubu lázeňského města Karlovy Vary",
    generatorname: "Metalsmith",
    generatorurl: "http://metalsmith.io/"
  })
  .source('./src/content')    // source directory
  .destination('./build')     // destination directory
  .clean(true)                // clean destination before
  .use(ignore(['.git/**']))
  .use(frontmatter({
    lang: 'toml',
    namespace: 'metadata',
    delims: ['```', '```'],
  }))
  .use(gitInfo({
    mergeMessage: [
      "See merge request ",
      "(Autoexportováno z Joomly)",
      "(Autoexportován z Joomly)",
      "[publish]"
    ]
  }))
  .use(function(files, metalsmith, done) {
    setImmediate(done);
    //console.log({files, metalsmith})
  })
  .use(collections({          // group all blog posts by internally
    articles: 'articles/*.md',// adding key 'collections':'posts'
  }))                         // use `collections.posts` in layouts
  .use(markdown())            // transpile all md into html
  .use(function(files,metalsmith, done){
    setImmediate(done)
    Object.keys(files).forEach(function(file) {
      files[file].content = files[file].contents.toString()
    })
  })
  .use((files,b,done) => {
    Object.keys(files).forEach((file) => {
      var metadata = files[file].metadata
      if(!Array.isArray(tags)) {
        metadata.tags = []
      }
      files[file].tags = metadata.tags
      if(!metadata.notarticle) {
        metadata.tags.push("Článek")
      }
    })
    done()
  })
  .use(tags({
    handle: 'tags',
    path: 'tag/:tag/index.html',
    pathPage: 'tag/:tag/:num/index.html',
    perPage: 6,
    layout: 'tag.html.nunj',
    sortBy: 'date',
    reverse: true,
  }))
  .use((files,b,done) => {
    Object.keys(files).forEach((file) => {
      if(!/^articles/.exec(file)) return
      if(!/\.html$/.exec(file)) return
      console.log({[file]: files[file] })
    })
    done()
  })
  .use(permalinks({           // change URLs to permalink URLs
    relative: false           // put css only in /css
  }))
  .use(layouts({              // wrap layouts around html
    engine: 'nunjucks',       // use the layout engine you like
    directory: './src/theme/templates',
    partials: './src/theme/templates/partials',
    default: 'article.html.nunj',
  }))
  .use(debug())
  .build(function(err) {      // build process
    if (err) throw err;       // error handling is required
  });
